@@ -9,9 +9,6 @@ namespace dxvk {
           D3D11Device*                      pDevice,
           D3D11ImmediateContext*            pContext)
   : m_device(pDevice), m_context(pContext) {
-    // Respecting the single-threaded flag may improve performance
-    UINT flags = pDevice->GetCreationFlags();
-    m_threadSafe = !(flags & D3D10_CREATE_DEVICE_SINGLETHREADED);
   }
 
   
@@ -717,8 +714,6 @@ namespace dxvk {
   void STDMETHODCALLTYPE D3D10Device::ClearRenderTargetView(
           ID3D10RenderTargetView*           pRenderTargetView,
     const FLOAT                             ColorRGBA[4]) {
-    D3D10DeviceLock lock = LockDevice();
-
     D3D10RenderTargetView* d3d10View = static_cast<D3D10RenderTargetView*>(pRenderTargetView);
     D3D11RenderTargetView* d3d11View = d3d10View ? d3d10View->GetD3D11Iface() : nullptr;
 
@@ -731,8 +726,6 @@ namespace dxvk {
           UINT                              ClearFlags,
           FLOAT                             Depth,
           UINT8                             Stencil) {
-    D3D10DeviceLock lock = LockDevice();
-
     D3D10DepthStencilView* d3d10View = static_cast<D3D10DepthStencilView*>(pDepthStencilView);
     D3D11DepthStencilView* d3d11View = d3d10View ? d3d10View->GetD3D11Iface() : nullptr;
 
@@ -743,8 +736,6 @@ namespace dxvk {
   void STDMETHODCALLTYPE D3D10Device::SetPredication(
           ID3D10Predicate*                  pPredicate,
           BOOL                              PredicateValue) {
-    D3D10DeviceLock lock = LockDevice();
-
     D3D10Query* d3d10Predicate = static_cast<D3D10Query*>(pPredicate);
     D3D11Query* d3d11Predicate = d3d10Predicate ? d3d10Predicate->GetD3D11Iface() : nullptr;
 
@@ -755,15 +746,14 @@ namespace dxvk {
   void STDMETHODCALLTYPE D3D10Device::GetPredication(
           ID3D10Predicate**                 ppPredicate,
           BOOL*                             pPredicateValue) {
-    D3D10DeviceLock lock = LockDevice();
     ID3D11Predicate* d3d11Predicate = nullptr;
 
     m_context->GetPredication(
       ppPredicate ? &d3d11Predicate : nullptr,
       pPredicateValue);
-    
+
     if (ppPredicate != nullptr)
-      *ppPredicate = static_cast<D3D11Query*>(d3d11Predicate)->GetD3D10Iface();
+      *ppPredicate = d3d11Predicate ? static_cast<D3D11Query*>(d3d11Predicate)->GetD3D10Iface() : nullptr;
   }
 
 
@@ -776,8 +766,6 @@ namespace dxvk {
           ID3D10Resource*                   pSrcResource,
           UINT                              SrcSubresource,
     const D3D10_BOX*                        pSrcBox) {
-    D3D10DeviceLock lock = LockDevice();
-
     Com<ID3D11Resource> d3d11DstResource;
     Com<ID3D11Resource> d3d11SrcResource;
     
@@ -794,8 +782,6 @@ namespace dxvk {
   void STDMETHODCALLTYPE D3D10Device::CopyResource(
           ID3D10Resource*                   pDstResource,
           ID3D10Resource*                   pSrcResource) {
-    D3D10DeviceLock lock = LockDevice();
-
     Com<ID3D11Resource> d3d11DstResource;
     Com<ID3D11Resource> d3d11SrcResource;
     
@@ -815,8 +801,6 @@ namespace dxvk {
     const void*                             pSrcData,
           UINT                              SrcRowPitch,
           UINT                              SrcDepthPitch) {
-    D3D10DeviceLock lock = LockDevice();
-
     Com<ID3D11Resource> d3d11DstResource;
     GetD3D11Resource(pDstResource, &d3d11DstResource);
 
@@ -829,8 +813,6 @@ namespace dxvk {
 
   void STDMETHODCALLTYPE D3D10Device::GenerateMips(
           ID3D10ShaderResourceView*         pShaderResourceView) {
-    D3D10DeviceLock lock = LockDevice();
-
     D3D10ShaderResourceView* d3d10View = static_cast<D3D10ShaderResourceView*>(pShaderResourceView);
     D3D11ShaderResourceView* d3d11View = d3d10View ? d3d10View->GetD3D11Iface() : nullptr;
 
@@ -844,8 +826,6 @@ namespace dxvk {
           ID3D10Resource*                   pSrcResource,
           UINT                              SrcSubresource,
           DXGI_FORMAT                       Format) {
-    D3D10DeviceLock lock = LockDevice();
-
     Com<ID3D11Resource> d3d11DstResource;
     Com<ID3D11Resource> d3d11SrcResource;
     
@@ -862,8 +842,6 @@ namespace dxvk {
   void STDMETHODCALLTYPE D3D10Device::Draw(
           UINT                              VertexCount,
           UINT                              StartVertexLocation) {
-    D3D10DeviceLock lock = LockDevice();
-
     m_context->Draw(VertexCount,
       StartVertexLocation);
   }
@@ -873,8 +851,6 @@ namespace dxvk {
           UINT                              IndexCount,
           UINT                              StartIndexLocation,
           INT                               BaseVertexLocation) {
-    D3D10DeviceLock lock = LockDevice();
-
     m_context->DrawIndexed(IndexCount,
       StartIndexLocation,
       BaseVertexLocation);
@@ -886,8 +862,6 @@ namespace dxvk {
           UINT                              InstanceCount,
           UINT                              StartVertexLocation,
           UINT                              StartInstanceLocation) {
-    D3D10DeviceLock lock = LockDevice();
-
     m_context->DrawInstanced(
       VertexCountPerInstance,
       InstanceCount,
@@ -902,8 +876,6 @@ namespace dxvk {
           UINT                              StartIndexLocation,
           INT                               BaseVertexLocation,
           UINT                              StartInstanceLocation) {
-    D3D10DeviceLock lock = LockDevice();
-
     m_context->DrawIndexedInstanced(
       IndexCountPerInstance,
       InstanceCount,
@@ -914,16 +886,12 @@ namespace dxvk {
 
 
   void STDMETHODCALLTYPE D3D10Device::DrawAuto() {
-    D3D10DeviceLock lock = LockDevice();
-
     m_context->DrawAuto();
   }
 
 
   void STDMETHODCALLTYPE D3D10Device::IASetInputLayout(
           ID3D10InputLayout*                pInputLayout) {
-    D3D10DeviceLock lock = LockDevice();
-
     D3D10InputLayout* d3d10InputLayout = static_cast<D3D10InputLayout*>(pInputLayout);
     D3D11InputLayout* d3d11InputLayout = d3d10InputLayout ? d3d10InputLayout->GetD3D11Iface() : nullptr;
 
@@ -933,8 +901,6 @@ namespace dxvk {
 
   void STDMETHODCALLTYPE D3D10Device::IASetPrimitiveTopology(
           D3D10_PRIMITIVE_TOPOLOGY          Topology) {
-    D3D10DeviceLock lock = LockDevice();
-
     m_context->IASetPrimitiveTopology(
       D3D11_PRIMITIVE_TOPOLOGY(Topology));
   }
@@ -946,8 +912,6 @@ namespace dxvk {
           ID3D10Buffer* const*              ppVertexBuffers,
     const UINT*                             pStrides,
     const UINT*                             pOffsets) {
-    D3D10DeviceLock lock = LockDevice();
-    
     ID3D11Buffer* d3d11Buffers[D3D10_IA_VERTEX_INPUT_RESOURCE_SLOT_COUNT];
 
     if (NumBuffers > D3D10_IA_VERTEX_INPUT_RESOURCE_SLOT_COUNT)
@@ -969,8 +933,6 @@ namespace dxvk {
           ID3D10Buffer*                     pIndexBuffer,
           DXGI_FORMAT                       Format,
           UINT                              Offset) {
-    D3D10DeviceLock lock = LockDevice();
-
     D3D10Buffer* d3d10Buffer = static_cast<D3D10Buffer*>(pIndexBuffer);
     D3D11Buffer* d3d11Buffer = d3d10Buffer ? d3d10Buffer->GetD3D11Iface() : nullptr;
 
@@ -981,19 +943,15 @@ namespace dxvk {
   void STDMETHODCALLTYPE D3D10Device::IAGetInputLayout(
           ID3D10InputLayout**               ppInputLayout) {
     ID3D11InputLayout* d3d11InputLayout = nullptr;
-    D3D10DeviceLock lock = LockDevice();
-
     m_context->IAGetInputLayout(&d3d11InputLayout);
 
-    *ppInputLayout = static_cast<D3D11InputLayout*>(
-      d3d11InputLayout)->GetD3D10Iface();
+    *ppInputLayout = d3d11InputLayout ? static_cast<D3D11InputLayout*>(
+      d3d11InputLayout)->GetD3D10Iface() : nullptr;
   }
 
 
   void STDMETHODCALLTYPE D3D10Device::IAGetPrimitiveTopology(
           D3D10_PRIMITIVE_TOPOLOGY*         pTopology) {
-    D3D10DeviceLock lock = LockDevice();
-
     D3D11_PRIMITIVE_TOPOLOGY d3d11Topology;
     m_context->IAGetPrimitiveTopology(&d3d11Topology);
 
@@ -1009,15 +967,13 @@ namespace dxvk {
           ID3D10Buffer**                    ppVertexBuffers,
           UINT*                             pStrides,
           UINT*                             pOffsets) {
-    D3D10DeviceLock lock = LockDevice();
-
     ID3D11Buffer* d3d11Buffers[D3D10_IA_VERTEX_INPUT_RESOURCE_SLOT_COUNT];
 
     m_context->IAGetVertexBuffers(
       StartSlot, NumBuffers,
       ppVertexBuffers ? d3d11Buffers : nullptr,
       pStrides, pOffsets);
-    
+
     if (ppVertexBuffers != nullptr) {
       for (uint32_t i = 0; i < NumBuffers; i++) {
         ppVertexBuffers[i] = d3d11Buffers[i]
@@ -1032,23 +988,19 @@ namespace dxvk {
           ID3D10Buffer**                    pIndexBuffer,
           DXGI_FORMAT*                      Format,
           UINT*                             Offset) {
-    D3D10DeviceLock lock = LockDevice();
-    
     ID3D11Buffer* d3d11Buffer = nullptr;
 
     m_context->IAGetIndexBuffer(
       pIndexBuffer ? &d3d11Buffer : nullptr,
       Format, Offset);
-    
+
     if (pIndexBuffer != nullptr)
-      *pIndexBuffer = static_cast<D3D11Buffer*>(d3d11Buffer)->GetD3D10Iface();
+      *pIndexBuffer = d3d11Buffer ? static_cast<D3D11Buffer*>(d3d11Buffer)->GetD3D10Iface() : nullptr;
   }
 
 
   void STDMETHODCALLTYPE D3D10Device::VSSetShader(
           ID3D10VertexShader*               pVertexShader) {
-    D3D10DeviceLock lock = LockDevice();
-
     D3D10VertexShader* d3d10Shader = static_cast<D3D10VertexShader*>(pVertexShader);
     D3D11VertexShader* d3d11Shader = d3d10Shader ? d3d10Shader->GetD3D11Iface() : nullptr;
 
@@ -1060,8 +1012,6 @@ namespace dxvk {
           UINT                              StartSlot,
           UINT                              NumBuffers,
           ID3D10Buffer* const*              ppConstantBuffers) {
-    D3D10DeviceLock lock = LockDevice();
-
     ID3D11Buffer* d3d11Buffers[D3D11_COMMONSHADER_CONSTANT_BUFFER_API_SLOT_COUNT];
 
     if (NumBuffers > D3D11_COMMONSHADER_CONSTANT_BUFFER_API_SLOT_COUNT)
@@ -1081,13 +1031,11 @@ namespace dxvk {
           UINT                              StartSlot,
           UINT                              NumViews,
           ID3D10ShaderResourceView* const*  ppShaderResourceViews) {
-    D3D10DeviceLock lock = LockDevice();
-
     ID3D11ShaderResourceView* d3d11Views[D3D10_COMMONSHADER_INPUT_RESOURCE_SLOT_COUNT];
 
     if (NumViews > D3D10_COMMONSHADER_INPUT_RESOURCE_SLOT_COUNT)
       return;
-    
+
     for (uint32_t i = 0; i < NumViews; i++) {
       d3d11Views[i] = ppShaderResourceViews && ppShaderResourceViews[i]
         ? static_cast<D3D10ShaderResourceView*>(ppShaderResourceViews[i])->GetD3D11Iface()
@@ -1102,8 +1050,6 @@ namespace dxvk {
           UINT                              StartSlot,
           UINT                              NumSamplers,
           ID3D10SamplerState* const*        ppSamplers) {
-    D3D10DeviceLock lock = LockDevice();
-
     ID3D11SamplerState* d3d11Samplers[D3D10_COMMONSHADER_SAMPLER_SLOT_COUNT];
 
     if (NumSamplers > D3D10_COMMONSHADER_SAMPLER_SLOT_COUNT)
@@ -1121,12 +1067,10 @@ namespace dxvk {
 
   void STDMETHODCALLTYPE D3D10Device::VSGetShader(
           ID3D10VertexShader**              ppVertexShader) {
-    D3D10DeviceLock lock = LockDevice();
-
     ID3D11VertexShader* d3d11Shader = nullptr;
     m_context->VSGetShader(&d3d11Shader, nullptr, nullptr);
 
-    *ppVertexShader = static_cast<D3D11VertexShader*>(d3d11Shader)->GetD3D10Iface();
+    *ppVertexShader = d3d11Shader ? static_cast<D3D11VertexShader*>(d3d11Shader)->GetD3D10Iface() : nullptr;
   }
 
 
@@ -1134,8 +1078,6 @@ namespace dxvk {
           UINT                              StartSlot,
           UINT                              NumBuffers,
           ID3D10Buffer**                    ppConstantBuffers) {
-    D3D10DeviceLock lock = LockDevice();
-
     ID3D11Buffer* d3d11Buffers[D3D11_COMMONSHADER_CONSTANT_BUFFER_API_SLOT_COUNT];
     m_context->VSGetConstantBuffers(StartSlot, NumBuffers, d3d11Buffers);
 
@@ -1151,8 +1093,6 @@ namespace dxvk {
           UINT                              StartSlot,
           UINT                              NumViews,
           ID3D10ShaderResourceView**        ppShaderResourceViews) {
-    D3D10DeviceLock lock = LockDevice();
-    
     ID3D11ShaderResourceView* d3d11Views[D3D10_COMMONSHADER_INPUT_RESOURCE_SLOT_COUNT];
     m_context->VSGetShaderResources(StartSlot, NumViews, d3d11Views);
 
@@ -1168,8 +1108,6 @@ namespace dxvk {
           UINT                              StartSlot,
           UINT                              NumSamplers,
           ID3D10SamplerState**              ppSamplers) {
-    D3D10DeviceLock lock = LockDevice();
-    
     ID3D11SamplerState* d3d11Samplers[D3D10_COMMONSHADER_SAMPLER_SLOT_COUNT];
     m_context->VSGetSamplers(StartSlot, NumSamplers, d3d11Samplers);
 
@@ -1183,8 +1121,6 @@ namespace dxvk {
 
   void STDMETHODCALLTYPE D3D10Device::GSSetShader(
           ID3D10GeometryShader*             pShader) {
-    D3D10DeviceLock lock = LockDevice();
-    
     D3D10GeometryShader* d3d10Shader = static_cast<D3D10GeometryShader*>(pShader);
     D3D11GeometryShader* d3d11Shader = d3d10Shader ? d3d10Shader->GetD3D11Iface() : nullptr;
 
@@ -1196,8 +1132,6 @@ namespace dxvk {
           UINT                              StartSlot,
           UINT                              NumBuffers,
           ID3D10Buffer* const*              ppConstantBuffers) {
-    D3D10DeviceLock lock = LockDevice();
-    
     ID3D11Buffer* d3d11Buffers[D3D11_COMMONSHADER_CONSTANT_BUFFER_API_SLOT_COUNT];
 
     if (NumBuffers > D3D11_COMMONSHADER_CONSTANT_BUFFER_API_SLOT_COUNT)
@@ -1217,8 +1151,6 @@ namespace dxvk {
           UINT                              StartSlot,
           UINT                              NumViews,
           ID3D10ShaderResourceView* const*  ppShaderResourceViews) {
-    D3D10DeviceLock lock = LockDevice();
-    
     ID3D11ShaderResourceView* d3d11Views[D3D10_COMMONSHADER_INPUT_RESOURCE_SLOT_COUNT];
 
     if (NumViews > D3D10_COMMONSHADER_INPUT_RESOURCE_SLOT_COUNT)
@@ -1238,8 +1170,6 @@ namespace dxvk {
           UINT                              StartSlot,
           UINT                              NumSamplers,
           ID3D10SamplerState* const*        ppSamplers) {
-    D3D10DeviceLock lock = LockDevice();
-    
     ID3D11SamplerState* d3d11Samplers[D3D10_COMMONSHADER_SAMPLER_SLOT_COUNT];
 
     if (NumSamplers > D3D10_COMMONSHADER_SAMPLER_SLOT_COUNT)
@@ -1257,12 +1187,10 @@ namespace dxvk {
 
   void STDMETHODCALLTYPE D3D10Device::GSGetShader(
           ID3D10GeometryShader**            ppGeometryShader) {
-    D3D10DeviceLock lock = LockDevice();
-    
     ID3D11GeometryShader* d3d11Shader = nullptr;
     m_context->GSGetShader(&d3d11Shader, nullptr, nullptr);
 
-    *ppGeometryShader = static_cast<D3D11GeometryShader*>(d3d11Shader)->GetD3D10Iface();
+    *ppGeometryShader = d3d11Shader ? static_cast<D3D11GeometryShader*>(d3d11Shader)->GetD3D10Iface() : nullptr;
   }
 
 
@@ -1270,8 +1198,6 @@ namespace dxvk {
           UINT                              StartSlot,
           UINT                              NumBuffers,
           ID3D10Buffer**                    ppConstantBuffers) {
-    D3D10DeviceLock lock = LockDevice();
-    
     ID3D11Buffer* d3d11Buffers[D3D11_COMMONSHADER_CONSTANT_BUFFER_API_SLOT_COUNT];
     m_context->GSGetConstantBuffers(StartSlot, NumBuffers, d3d11Buffers);
 
@@ -1287,8 +1213,6 @@ namespace dxvk {
           UINT                              StartSlot,
           UINT                              NumViews,
           ID3D10ShaderResourceView**        ppShaderResourceViews) {
-    D3D10DeviceLock lock = LockDevice();
-    
     ID3D11ShaderResourceView* d3d11Views[D3D10_COMMONSHADER_INPUT_RESOURCE_SLOT_COUNT];
     m_context->GSGetShaderResources(StartSlot, NumViews, d3d11Views);
 
@@ -1304,8 +1228,6 @@ namespace dxvk {
           UINT                              StartSlot,
           UINT                              NumSamplers,
           ID3D10SamplerState**              ppSamplers) {
-    D3D10DeviceLock lock = LockDevice();
-    
     ID3D11SamplerState* d3d11Samplers[D3D10_COMMONSHADER_SAMPLER_SLOT_COUNT];
     m_context->GSGetSamplers(StartSlot, NumSamplers, d3d11Samplers);
 
@@ -1319,8 +1241,6 @@ namespace dxvk {
 
   void STDMETHODCALLTYPE D3D10Device::PSSetShader(
           ID3D10PixelShader*                pPixelShader) {
-    D3D10DeviceLock lock = LockDevice();
-    
     D3D10PixelShader* d3d10Shader = static_cast<D3D10PixelShader*>(pPixelShader);
     D3D11PixelShader* d3d11Shader = d3d10Shader ? d3d10Shader->GetD3D11Iface() : nullptr;
 
@@ -1332,8 +1252,6 @@ namespace dxvk {
           UINT                              StartSlot,
           UINT                              NumBuffers,
           ID3D10Buffer* const*              ppConstantBuffers) {
-    D3D10DeviceLock lock = LockDevice();
-    
     ID3D11Buffer* d3d11Buffers[D3D11_COMMONSHADER_CONSTANT_BUFFER_API_SLOT_COUNT];
 
     if (NumBuffers > D3D11_COMMONSHADER_CONSTANT_BUFFER_API_SLOT_COUNT)
@@ -1353,8 +1271,6 @@ namespace dxvk {
           UINT                              StartSlot,
           UINT                              NumViews,
           ID3D10ShaderResourceView* const*  ppShaderResourceViews) {
-    D3D10DeviceLock lock = LockDevice();
-    
     ID3D11ShaderResourceView* d3d11Views[D3D10_COMMONSHADER_INPUT_RESOURCE_SLOT_COUNT];
 
     if (NumViews > D3D10_COMMONSHADER_INPUT_RESOURCE_SLOT_COUNT)
@@ -1374,8 +1290,6 @@ namespace dxvk {
           UINT                              StartSlot,
           UINT                              NumSamplers,
           ID3D10SamplerState* const*        ppSamplers) {
-    D3D10DeviceLock lock = LockDevice();
-    
     ID3D11SamplerState* d3d11Samplers[D3D10_COMMONSHADER_SAMPLER_SLOT_COUNT];
 
     if (NumSamplers > D3D10_COMMONSHADER_SAMPLER_SLOT_COUNT)
@@ -1393,12 +1307,10 @@ namespace dxvk {
 
   void STDMETHODCALLTYPE D3D10Device::PSGetShader(
           ID3D10PixelShader**               ppPixelShader) {
-    D3D10DeviceLock lock = LockDevice();
-    
     ID3D11PixelShader* d3d11Shader = nullptr;
     m_context->PSGetShader(&d3d11Shader, nullptr, nullptr);
 
-    *ppPixelShader = static_cast<D3D11PixelShader*>(d3d11Shader)->GetD3D10Iface();
+    *ppPixelShader = d3d11Shader ? static_cast<D3D11PixelShader*>(d3d11Shader)->GetD3D10Iface() : nullptr;
   }
 
 
@@ -1406,8 +1318,6 @@ namespace dxvk {
           UINT                              StartSlot,
           UINT                              NumBuffers,
           ID3D10Buffer**                    ppConstantBuffers) {
-    D3D10DeviceLock lock = LockDevice();
-    
     ID3D11Buffer* d3d11Buffers[D3D11_COMMONSHADER_CONSTANT_BUFFER_API_SLOT_COUNT];
     m_context->PSGetConstantBuffers(StartSlot, NumBuffers, d3d11Buffers);
 
@@ -1423,8 +1333,6 @@ namespace dxvk {
           UINT                              StartSlot,
           UINT                              NumViews,
           ID3D10ShaderResourceView**        ppShaderResourceViews) {
-    D3D10DeviceLock lock = LockDevice();
-    
     ID3D11ShaderResourceView* d3d11Views[D3D10_COMMONSHADER_INPUT_RESOURCE_SLOT_COUNT];
     m_context->PSGetShaderResources(StartSlot, NumViews, d3d11Views);
 
@@ -1440,8 +1348,6 @@ namespace dxvk {
           UINT                              StartSlot,
           UINT                              NumSamplers,
           ID3D10SamplerState**              ppSamplers) {
-    D3D10DeviceLock lock = LockDevice();
-    
     ID3D11SamplerState* d3d11Samplers[D3D10_COMMONSHADER_SAMPLER_SLOT_COUNT];
     m_context->PSGetSamplers(StartSlot, NumSamplers, d3d11Samplers);
 
@@ -1457,8 +1363,6 @@ namespace dxvk {
           UINT                              NumViews,
           ID3D10RenderTargetView* const*    ppRenderTargetViews,
           ID3D10DepthStencilView*           pDepthStencilView) {
-    D3D10DeviceLock lock = LockDevice();
-    
     ID3D11RenderTargetView* d3d11Rtv[D3D10_SIMULTANEOUS_RENDER_TARGET_COUNT];
 
     if (NumViews > D3D10_SIMULTANEOUS_RENDER_TARGET_COUNT)
@@ -1481,8 +1385,6 @@ namespace dxvk {
           ID3D10BlendState*                 pBlendState,
     const FLOAT                             BlendFactor[4],
           UINT                              SampleMask) {
-    D3D10DeviceLock lock = LockDevice();
-    
     D3D10BlendState* d3d10BlendState = static_cast<D3D10BlendState*>(pBlendState);
     D3D11BlendState* d3d11BlendState = d3d10BlendState ? d3d10BlendState->GetD3D11Iface() : nullptr;
 
@@ -1504,15 +1406,13 @@ namespace dxvk {
           UINT                              NumViews,
           ID3D10RenderTargetView**          ppRenderTargetViews,
           ID3D10DepthStencilView**          ppDepthStencilView) {
-    D3D10DeviceLock lock = LockDevice();
-    
     ID3D11RenderTargetView* d3d11Rtv[D3D10_SIMULTANEOUS_RENDER_TARGET_COUNT];
-    ID3D11DepthStencilView* d3d11Dsv;
+    ID3D11DepthStencilView* d3d11Dsv = nullptr;
 
     m_context->OMGetRenderTargets(NumViews,
       ppRenderTargetViews ? d3d11Rtv : nullptr,
       ppDepthStencilView ? &d3d11Dsv : nullptr);
-    
+
     if (ppRenderTargetViews != nullptr) {
       for (uint32_t i = 0; i < NumViews; i++) {
         ppRenderTargetViews[i] = d3d11Rtv[i]
@@ -1522,7 +1422,7 @@ namespace dxvk {
     }
 
     if (ppDepthStencilView)
-      *ppDepthStencilView = static_cast<D3D11DepthStencilView*>(d3d11Dsv)->GetD3D10Iface();
+      *ppDepthStencilView = d3d11Dsv ? static_cast<D3D11DepthStencilView*>(d3d11Dsv)->GetD3D10Iface() : nullptr;
   }
 
 
@@ -1530,8 +1430,6 @@ namespace dxvk {
           ID3D10BlendState**                ppBlendState,
           FLOAT                             BlendFactor[4],
           UINT*                             pSampleMask) {
-    D3D10DeviceLock lock = LockDevice();
-    
     ID3D11BlendState* d3d11BlendState = nullptr;
 
     m_context->OMGetBlendState(
@@ -1539,15 +1437,13 @@ namespace dxvk {
       BlendFactor, pSampleMask);
 
     if (ppBlendState != nullptr)
-      *ppBlendState = static_cast<D3D11BlendState*>(d3d11BlendState)->GetD3D10Iface();
+      *ppBlendState = d3d11BlendState ? static_cast<D3D11BlendState*>(d3d11BlendState)->GetD3D10Iface() : nullptr;
   }
 
 
   void STDMETHODCALLTYPE D3D10Device::OMGetDepthStencilState(
           ID3D10DepthStencilState**         ppDepthStencilState,
           UINT*                             pStencilRef) {
-    D3D10DeviceLock lock = LockDevice();
-    
     ID3D11DepthStencilState* d3d11DepthStencilState = nullptr;
 
     m_context->OMGetDepthStencilState(
@@ -1555,14 +1451,12 @@ namespace dxvk {
       pStencilRef);
 
     if (ppDepthStencilState != nullptr)
-      *ppDepthStencilState = static_cast<D3D11DepthStencilState*>(d3d11DepthStencilState)->GetD3D10Iface();
+      *ppDepthStencilState = d3d11DepthStencilState ? static_cast<D3D11DepthStencilState*>(d3d11DepthStencilState)->GetD3D10Iface() : nullptr;
   }
 
 
   void STDMETHODCALLTYPE D3D10Device::RSSetState(
           ID3D10RasterizerState*            pRasterizerState) {
-    D3D10DeviceLock lock = LockDevice();
-    
     D3D10RasterizerState* d3d10RasterizerState = static_cast<D3D10RasterizerState*>(pRasterizerState);
     D3D11RasterizerState* d3d11RasterizerState = d3d10RasterizerState ? d3d10RasterizerState->GetD3D11Iface() : nullptr;
 
@@ -1573,8 +1467,6 @@ namespace dxvk {
   void STDMETHODCALLTYPE D3D10Device::RSSetViewports(
           UINT                              NumViewports,
     const D3D10_VIEWPORT*                   pViewports) {
-    D3D10DeviceLock lock = LockDevice();
-    
     D3D11_VIEWPORT vp[D3D10_VIEWPORT_AND_SCISSORRECT_OBJECT_COUNT_PER_PIPELINE];
 
     if (NumViewports > D3D10_VIEWPORT_AND_SCISSORRECT_OBJECT_COUNT_PER_PIPELINE)
@@ -1596,28 +1488,22 @@ namespace dxvk {
   void STDMETHODCALLTYPE D3D10Device::RSSetScissorRects(
           UINT                              NumRects,
     const D3D10_RECT*                       pRects) {
-    D3D10DeviceLock lock = LockDevice();
-    
     m_context->RSSetScissorRects(NumRects, pRects);
   }
 
 
   void STDMETHODCALLTYPE D3D10Device::RSGetState(
           ID3D10RasterizerState**           ppRasterizerState) {
-    D3D10DeviceLock lock = LockDevice();
-    
     ID3D11RasterizerState* d3d11RasterizerState = nullptr;
     m_context->RSGetState(&d3d11RasterizerState);
 
-    *ppRasterizerState = static_cast<D3D11RasterizerState*>(d3d11RasterizerState)->GetD3D10Iface();
+    *ppRasterizerState = d3d11RasterizerState ? static_cast<D3D11RasterizerState*>(d3d11RasterizerState)->GetD3D10Iface() : nullptr;
   }
 
 
   void STDMETHODCALLTYPE D3D10Device::RSGetViewports(
           UINT*                             NumViewports,
           D3D10_VIEWPORT*                   pViewports) {
-    D3D10DeviceLock lock = LockDevice();
-    
     D3D11_VIEWPORT vp[D3D10_VIEWPORT_AND_SCISSORRECT_OBJECT_COUNT_PER_PIPELINE];
     m_context->RSGetViewports(NumViewports, pViewports != nullptr ? vp : nullptr);
 
@@ -1637,8 +1523,6 @@ namespace dxvk {
   void STDMETHODCALLTYPE D3D10Device::RSGetScissorRects(
           UINT*                             NumRects,
           D3D10_RECT*                       pRects) {
-    D3D10DeviceLock lock = LockDevice();
-    
     m_context->RSGetScissorRects(NumRects, pRects);
   }
 
@@ -1647,8 +1531,6 @@ namespace dxvk {
           UINT                              NumBuffers,
           ID3D10Buffer* const*              ppSOTargets,
     const UINT*                             pOffsets) {
-    D3D10DeviceLock lock = LockDevice();
-    
     ID3D11Buffer* d3d11Buffers[D3D10_SO_BUFFER_SLOT_COUNT];
 
     if (NumBuffers > D3D10_SO_BUFFER_SLOT_COUNT)
@@ -1663,13 +1545,11 @@ namespace dxvk {
     m_context->SOSetTargets(NumBuffers, d3d11Buffers, pOffsets);
   }
 
-  
+
   void STDMETHODCALLTYPE D3D10Device::SOGetTargets(
           UINT                              NumBuffers,
           ID3D10Buffer**                    ppSOTargets,
           UINT*                             pOffsets) {
-    D3D10DeviceLock lock = LockDevice();
-    
     ID3D11Buffer* d3d11Buffers[D3D10_SO_BUFFER_SLOT_COUNT];
 
     m_context->SOGetTargetsWithOffsets(NumBuffers,
