@@ -4,7 +4,10 @@
 #include "../dxvk/dxvk_cs.h"
 #include "../dxvk/dxvk_device.h"
 
+#include "../d3d10/d3d10_multithread.h"
+
 #include "d3d11_annotation.h"
+#include "d3d11_cmd.h"
 #include "d3d11_context_state.h"
 #include "d3d11_device_child.h"
 #include "d3d11_texture.h"
@@ -642,11 +645,12 @@ namespace dxvk {
       const VkImageSubresourceRange*  pSubresources,
             VkImageLayout             OldLayout,
             VkImageLayout             NewLayout);
-    
+
   protected:
     
     D3D11Device* const          m_parent;
     D3D11UserDefinedAnnotation  m_annotation;
+    D3D10Multithread            m_multithread;
     
     Rc<DxvkDevice>              m_device;
     Rc<DxvkDataBuffer>          m_updateBuffer;
@@ -659,6 +663,7 @@ namespace dxvk {
     Com<D3D11RasterizerState>   m_defaultRasterizerState;
     
     D3D11ContextState           m_state;
+    D3D11CmdData*               m_cmdData;
     
     void ApplyInputLayout();
     
@@ -716,8 +721,8 @@ namespace dxvk {
     
     void BindUnorderedAccessView(
             UINT                              UavSlot,
-            UINT                              CtrSlot,
             D3D11UnorderedAccessView*         pUav,
+            UINT                              CtrSlot,
             UINT                              Counter);
     
     void DiscardBuffer(
@@ -805,8 +810,14 @@ namespace dxvk {
       return pShader != nullptr ? pShader->GetCommonShader() : nullptr;
     }
 
+    D3D10DeviceLock LockContext() {
+      return m_multithread.AcquireLock();
+    }
+
     template<typename Cmd>
     void EmitCs(Cmd&& command) {
+      m_cmdData = nullptr;
+
       if (!m_csChunk->push(command)) {
         EmitCsChunk(std::move(m_csChunk));
         
@@ -814,11 +825,29 @@ namespace dxvk {
         m_csChunk->push(command);
       }
     }
+
+    template<typename M, typename Cmd, typename... Args>
+    M* EmitCsCmd(Cmd&& command, Args&&... args) {
+      M* data = m_csChunk->pushCmd<M, Cmd, Args...>(
+        command, std::forward<Args>(args)...);
+
+      if (!data) {
+        EmitCsChunk(std::move(m_csChunk));
+        
+        m_csChunk = AllocCsChunk();
+        data = m_csChunk->pushCmd<M, Cmd, Args...>(
+          command, std::forward<Args>(args)...);
+      }
+
+      m_cmdData = data;
+      return data;
+    }
     
     void FlushCsChunk() {
       if (m_csChunk->commandCount() != 0) {
         EmitCsChunk(std::move(m_csChunk));
         m_csChunk = AllocCsChunk();
+        m_cmdData = nullptr;
       }
     }
     
