@@ -42,6 +42,8 @@ namespace dxvk {
   void STDMETHODCALLTYPE D3D11DeferredContext::ExecuteCommandList(
           ID3D11CommandList*  pCommandList,
           BOOL                RestoreContextState) {
+    D3D10DeviceLock lock = LockContext();
+
     FlushCsChunk();
     
     static_cast<D3D11CommandList*>(pCommandList)->EmitToCommandList(m_commandList.ptr());
@@ -56,6 +58,8 @@ namespace dxvk {
   HRESULT STDMETHODCALLTYPE D3D11DeferredContext::FinishCommandList(
           BOOL                RestoreDeferredContextState,
           ID3D11CommandList   **ppCommandList) {
+    D3D10DeviceLock lock = LockContext();
+
     FlushCsChunk();
     
     if (ppCommandList != nullptr)
@@ -78,6 +82,8 @@ namespace dxvk {
           D3D11_MAP                   MapType,
           UINT                        MapFlags,
           D3D11_MAPPED_SUBRESOURCE*   pMappedResource) {
+    D3D10DeviceLock lock = LockContext();
+
     D3D11_RESOURCE_DIMENSION resourceDim = D3D11_RESOURCE_DIMENSION_UNKNOWN;
     pResource->GetType(&resourceDim);
     
@@ -132,6 +138,8 @@ namespace dxvk {
   void STDMETHODCALLTYPE D3D11DeferredContext::Unmap(
           ID3D11Resource*             pResource,
           UINT                        Subresource) {
+    D3D10DeviceLock lock = LockContext();
+
     D3D11_RESOURCE_DIMENSION resourceDim = D3D11_RESOURCE_DIMENSION_UNKNOWN;
     pResource->GetType(&resourceDim);
     
@@ -173,9 +181,9 @@ namespace dxvk {
     if (pBuffer->Desc()->Usage == D3D11_USAGE_DYNAMIC && m_csFlags.test(DxvkCsChunkFlag::SingleUse)) {
       // For resources that cannot be written by the GPU,
       // we may write to the buffer resource directly and
-      // just swap in the physical buffer slice as needed.
-      pMapEntry->BufferSlice = buffer->allocPhysicalSlice();
-      pMapEntry->MapPointer  = pMapEntry->BufferSlice.mapPtr(0);
+      // just swap in the buffer slice as needed.
+      pMapEntry->BufferSlice = buffer->allocSlice();
+      pMapEntry->MapPointer  = pMapEntry->BufferSlice.mapPtr;
     } else {
       // For GPU-writable resources, we need a data slice
       // to perform the update operation at execution time.
@@ -249,8 +257,8 @@ namespace dxvk {
         cDstBuffer = pBuffer->GetBuffer(),
         cDataSlice = pMapEntry->DataSlice
       ] (DxvkContext* ctx) {
-        DxvkPhysicalBufferSlice slice = cDstBuffer->allocPhysicalSlice();
-        std::memcpy(slice.mapPtr(0), cDataSlice.ptr(), cDataSlice.length());
+        DxvkBufferSliceHandle slice = cDstBuffer->allocSlice();
+        std::memcpy(slice.mapPtr, cDataSlice.ptr(), cDataSlice.length());
         ctx->invalidateBuffer(cDstBuffer, slice);
       });
     }
