@@ -8,14 +8,6 @@
 
 namespace dxvk {
   
-  /**
-   * \brief Sync mode
-   */
-  enum class D3D11SwapChainSyncMode : int32_t {
-    Default   = 0,
-    Mailbox   = 1,
-  };
-  
   struct D3D11Options {
     D3D11Options(const Config& config);
     /// Handle D3D11_MAP_FLAG_DO_NOT_WAIT properly.
@@ -33,14 +25,6 @@ namespace dxvk {
     /// than once.
     bool dcSingleUseMode;
 
-    /// Fakes stream output support.
-    /// 
-    /// Temporary hack that fixes issues in some games
-    /// which technically need stream output but work
-    /// well enough without it. Will be removed once
-    /// Stream Output is properly supported in DXVK.
-    bool fakeStreamOutSupport;
-
     /// Zero-initialize workgroup memory
     ///
     /// Workargound for games that don't initialize
@@ -68,13 +52,14 @@ namespace dxvk {
     /// passed to IDXGISwapChain::Present.
     int32_t syncInterval;
 
+    /// Override maximum frame latency if the app specifies
+    /// a higher value. May help with frame timing issues.
+    int32_t maxFrameLatency;
+
     /// Defer surface creation until first present call. This
     /// fixes issues with games that create multiple swap chains
     /// for a single window that may interfere with each other.
     bool deferSurfaceCreation;
-
-    /// Vsync mode
-    D3D11SwapChainSyncMode syncMode;
   };
   
 }
\ No newline at end of file
