@@ -178,6 +178,7 @@ namespace dxvk {
     uint32_t builtinLayer         = 0;
     uint32_t builtinViewportId    = 0;
     
+    uint32_t invocationMask       = 0;
     uint32_t killState            = 0;
   };
   
@@ -376,6 +377,7 @@ namespace dxvk {
       const DxbcProgramInfo&    programInfo,
       const Rc<DxbcIsgn>&       isgn,
       const Rc<DxbcIsgn>&       osgn,
+      const Rc<DxbcIsgn>&       psgn,
       const DxbcAnalysisInfo&   analysis);
     ~DxbcCompiler();
     
@@ -409,6 +411,7 @@ namespace dxvk {
     
     Rc<DxbcIsgn>        m_isgn;
     Rc<DxbcIsgn>        m_osgn;
+    Rc<DxbcIsgn>        m_psgn;
     
     const DxbcAnalysisInfo* m_analysis;
     
@@ -919,6 +922,9 @@ namespace dxvk {
     
     //////////////////////////
     // Resource query methods
+    DxbcRegisterValue emitQueryBufferSize(
+      const DxbcRegister&           resource);
+    
     DxbcRegisterValue emitQueryTexelBufferSize(
       const DxbcRegister&           resource);
     
