@@ -10,29 +10,31 @@ namespace dxvk {
 
 
   DxbcOptions::DxbcOptions(const Rc<DxvkDevice>& device, const D3D11Options& options) {
+    const Rc<DxvkAdapter> adapter = device->adapter();
+
     const DxvkDeviceFeatures& devFeatures = device->features();
-    const DxvkDeviceInfo& devInfo = device->adapter()->devicePropertiesExt();
+    const DxvkDeviceInfo& devInfo = adapter->devicePropertiesExt();
     
     useStorageImageReadWithoutFormat
       = devFeatures.core.features.shaderStorageImageReadWithoutFormat;
     useSubgroupOpsForEarlyDiscard
       = (devInfo.coreSubgroup.subgroupSize >= 4)
      && (devInfo.coreSubgroup.supportedStages     & VK_SHADER_STAGE_FRAGMENT_BIT)
-     && (devInfo.coreSubgroup.supportedOperations & VK_SUBGROUP_FEATURE_ARITHMETIC_BIT);
-    useSubgroupOpsClustered = useSubgroupOpsForEarlyDiscard
-     && (devInfo.coreSubgroup.supportedOperations & VK_SUBGROUP_FEATURE_CLUSTERED_BIT);
+     && (devInfo.coreSubgroup.supportedOperations & VK_SUBGROUP_FEATURE_BALLOT_BIT);
+    useRawSsbo
+      = (devInfo.core.properties.limits.minStorageBufferOffsetAlignment <= sizeof(uint32_t));
     
     zeroInitWorkgroupMemory = options.zeroInitWorkgroupMemory;
     
-    // Disable early discard on AMD due to GPU hangs
+    // Disable early discard on RADV due to GPU hangs
     // Disable early discard on Nvidia because it may hurt performance
-    auto vendor = DxvkGpuVendor(devInfo.core.properties.vendorID);
-
-    if (vendor == DxvkGpuVendor::Amd
-     || vendor == DxvkGpuVendor::Nvidia) {
+    if (adapter->matchesDriver(DxvkGpuVendor::Amd,    VK_DRIVER_ID_MESA_RADV_KHR,          0, 0)
+     || adapter->matchesDriver(DxvkGpuVendor::Nvidia, VK_DRIVER_ID_NVIDIA_PROPRIETARY_KHR, 0, 0))
       useSubgroupOpsForEarlyDiscard = false;
-      useSubgroupOpsClustered       = false;
-    }
+    
+    // Apply shader-related options
+    applyTristate(useSubgroupOpsForEarlyDiscard, device->config().useEarlyDiscard);
+    applyTristate(useRawSsbo,                    device->config().useRawSsbo);
   }
   
 }
\ No newline at end of file
