@@ -17,8 +17,9 @@ namespace dxvk {
     /// shader invocations if derivatives remain valid.
     bool useSubgroupOpsForEarlyDiscard = false;
 
-    /// Use clustered subgroup operations
-    bool useSubgroupOpsClustered = false;
+    /// Use SSBOs instead of texel buffers
+    /// for raw and structured buffers.
+    bool useRawSsbo = false;
 
     /// Clear thread-group shared memory to zero
     bool zeroInitWorkgroupMemory = false;
