@@ -62,25 +62,6 @@ IDXGIVkSwapChain : public IUnknown {
 };
 
 
-/**
- * \brief Private DXGI device interface
- * 
- * The implementation of \c IDXGIDevice stores a
- * \ref DxvkDevice which can be retrieved using
- * this interface.
- */
-MIDL_INTERFACE("7a622cf6-627a-46b2-b52f-360ef3da831c")
-IDXGIVkDevice : public IDXGIDevice3 {
-  static const GUID guid;
-  
-  virtual ~IDXGIVkDevice() { }
-  
-  virtual dxvk::Rc<dxvk::DxvkDevice> STDMETHODCALLTYPE GetDXVKDevice() = 0;
-  
-  virtual dxvk::Rc<dxvk::DxvkEvent> STDMETHODCALLTYPE GetFrameSyncEvent() = 0;
-};
-
-
 /**
  * \brief Private DXGI adapter interface
  * 
@@ -94,59 +75,6 @@ IDXGIVkAdapter : public IDXGIAdapter3 {
   
   virtual dxvk::Rc<dxvk::DxvkAdapter> STDMETHODCALLTYPE GetDXVKAdapter() = 0;
   
-  /**
-   * \brief Creates a DXGI device object
-   * 
-   * \param [in] pAdapter The adapter
-   * \param [in] pFeatures Device features to enable
-   * \param [out] ppDevice The DXGI device object
-   * \returns \c S_OK on success, or an error code
-   */
-  virtual HRESULT STDMETHODCALLTYPE CreateDevice(
-          IDXGIObject*              pContainer,
-    const dxvk::DxvkDeviceFeatures* pFeatures,
-          IDXGIVkDevice**           ppDevice) = 0;
-  
-  /**
-   * \brief Maps a DXGI format to a compatible Vulkan format
-   * 
-   * For color formats, the returned Vulkan format has the
-   * same memory layout as the DXGI format so that it can
-   * be mapped and copied to buffers. For depth-stencil
-   * formats, this is not guaranteed.
-   * \param [in] format The DXGI format
-   * \param [in] mode Format lookup mode
-   * \returns Vulkan format mapping
-   */
-  virtual dxvk::DXGI_VK_FORMAT_INFO STDMETHODCALLTYPE LookupFormat(
-          DXGI_FORMAT               Format,
-          dxvk::DXGI_VK_FORMAT_MODE Mode) = 0;
-  
-  /**
-   * \brief Queries the compatibility family of a given format
-   * 
-   * \param [in] Format The DXGI format
-   * \param [in] Mode Format lookup mode
-   * \returns Format family
-   */
-  virtual dxvk::DXGI_VK_FORMAT_FAMILY STDMETHODCALLTYPE LookupFormatFamily(
-          DXGI_FORMAT               Format,
-          dxvk::DXGI_VK_FORMAT_MODE Mode) = 0;
-};
-
-
-/**
- * \brief Swap chain back buffer interface
- * 
- * Allows the swap chain and presenter to query
- * the underlying image while it is embedded in
- * a texture object specified by the client API.
- */
-MIDL_INTERFACE("5679becd-8547-4d93-96a1-e61a1ce7ef37")
-IDXGIVkBackBuffer : public IUnknown {
-  static const GUID guid;
-  
-  virtual dxvk::Rc<dxvk::DxvkImage> GetDXVKImage() = 0;
 };
 
 
@@ -309,20 +237,38 @@ IDXGIVkInteropDevice : public IUnknown {
 };
 
 
+/**
+ * \brief IWineDXGISwapChainFactory device interface
+ *
+ * Allows a swap chain to be created from a device.
+ * See include/wine/winedxgi.idl for definition.
+ */
+MIDL_INTERFACE("53cb4ff0-c25a-4164-a891-0e83db0a7aac")
+IWineDXGISwapChainFactory : public IUnknown {
+    static const GUID guid;
+
+    virtual HRESULT STDMETHODCALLTYPE CreateSwapChainForHwnd(
+            IDXGIFactory*           pFactory,
+            HWND                    hWnd,
+      const DXGI_SWAP_CHAIN_DESC1*  pDesc,
+      const DXGI_SWAP_CHAIN_FULLSCREEN_DESC* pFullscreenDesc,
+            IDXGIOutput*            pRestrictToOutput,
+            IDXGISwapChain1**       ppSwapChain) = 0;
+};
+
+
 #ifdef _MSC_VER
 struct __declspec(uuid("907bf281-ea3c-43b4-a8e4-9f231107b4ff")) IDXGIVkAdapter;
-struct __declspec(uuid("7a622cf6-627a-46b2-b52f-360ef3da831c")) IDXGIVkDevice;
-struct __declspec(uuid("5679becd-8547-4d93-96a1-e61a1ce7ef37")) IDXGIVkBackBuffer;
 struct __declspec(uuid("79352328-16f2-4f81-9746-9c2e2ccd43cf")) IDXGIVkPresentDevice;
 struct __declspec(uuid("e2ef5fa5-dc21-4af7-90c4-f67ef6a09323")) IDXGIVkInteropDevice;
 struct __declspec(uuid("5546cf8c-77e7-4341-b05d-8d4d5000e77d")) IDXGIVkInteropSurface;
 struct __declspec(uuid("104001a6-7f36-4957-b932-86ade9567d91")) IDXGIVkSwapChain;
+struct __declspec(uuid("53cb4ff0-c25a-4164-a891-0e83db0a7aac")) IWineDXGISwapChainFactory;
 #else
 DXVK_DEFINE_GUID(IDXGIVkAdapter);
-DXVK_DEFINE_GUID(IDXGIVkDevice);
-DXVK_DEFINE_GUID(IDXGIVkBackBuffer);
 DXVK_DEFINE_GUID(IDXGIVkPresentDevice);
 DXVK_DEFINE_GUID(IDXGIVkInteropDevice);
 DXVK_DEFINE_GUID(IDXGIVkInteropSurface);
 DXVK_DEFINE_GUID(IDXGIVkSwapChain);
+DXVK_DEFINE_GUID(IWineDXGISwapChainFactory);
 #endif
\ No newline at end of file
