@@ -17,15 +17,6 @@ namespace dxvk {
   struct DxgiOptions {
     DxgiOptions(const Config& config);
 
-    /// Defer surface creation until first present call. This
-    /// fixes issues with games that create multiple swap chains
-    /// for a single window that may interfere with each other.
-    bool deferSurfaceCreation;
-
-    /// Override maximum frame latency if the app specifies
-    /// a higher value. May help with frame timing issues.
-    int32_t maxFrameLatency;
-
     /// Override PCI vendor and device IDs reported to the
     /// application. This may make apps think they are running
     /// on a different GPU than they do and behave differently.
@@ -40,6 +31,9 @@ namespace dxvk {
 
     /// Enables D3D10 support
     bool d3d10Enable;
+
+    /// Enables nvapi workaround
+    bool nvapiHack;
   };
   
 }
