@@ -1,5 +1,6 @@
 #pragma once
 
+#include "dxgi_monitor.h"
 #include "dxgi_object.h"
 
 namespace dxvk {
@@ -123,20 +124,11 @@ namespace dxvk {
             IUnknown*             pConcernedDevice,
             UINT*                 pFlags) final;
     
-    HRESULT GetDisplayMode(
-            DXGI_MODE_DESC*       pMode,
-            DWORD                 ModeNum);
-    
-    HRESULT SetDisplayMode(
-      const DXGI_MODE_DESC*       pMode);
-    
   private:
     
     Com<DxgiAdapter> m_adapter = nullptr;
     HMONITOR         m_monitor = nullptr;
     
-    uint32_t GetFormatBpp(DXGI_FORMAT Format) const;
-    
   };
 
 }
