@@ -1,4 +1,3 @@
-#include "dxgi_device.h"
 #include "dxgi_factory.h"
 #include "dxgi_output.h"
 #include "dxgi_swapchain.h"
@@ -6,16 +5,17 @@
 namespace dxvk {
   
   DxgiSwapChain::DxgiSwapChain(
-          DxgiFactory*                pFactory,
-          IUnknown*                   pDevice,
+          IDXGIFactory*               pFactory,
+          IDXGIVkSwapChain*           pPresenter,
           HWND                        hWnd,
     const DXGI_SWAP_CHAIN_DESC1*      pDesc,
     const DXGI_SWAP_CHAIN_FULLSCREEN_DESC*  pFullscreenDesc)
-  : m_factory (pFactory),
-    m_window  (hWnd),
-    m_desc    (*pDesc),
-    m_descFs  (*pFullscreenDesc),
-    m_monitor (nullptr) {
+  : m_factory   (pFactory),
+    m_window    (hWnd),
+    m_desc      (*pDesc),
+    m_descFs    (*pFullscreenDesc),
+    m_presenter (pPresenter),
+    m_monitor   (nullptr) {
     // Initialize frame statistics
     m_stats.PresentCount         = 0;
     m_stats.PresentRefreshCount  = 0;
@@ -23,31 +23,27 @@ namespace dxvk {
     m_stats.SyncQPCTime.QuadPart = 0;
     m_stats.SyncGPUTime.QuadPart = 0;
     
-    // Adjust initial back buffer size. If zero, these
-    // shall be set to the current window size.
-    const VkExtent2D windowSize = GetWindowSize();
-    
-    if (m_desc.Width  == 0) m_desc.Width  = windowSize.width;
-    if (m_desc.Height == 0) m_desc.Height = windowSize.height;
-    
-    // Create presenter, which also serves as an interface to the device
-    if (FAILED(CreatePresenter(pDevice, &m_presenter)))
-      throw DxvkError("DXGI: Failed to create presenter");
-    
     if (FAILED(m_presenter->GetAdapter(__uuidof(IDXGIAdapter), reinterpret_cast<void**>(&m_adapter))))
       throw DxvkError("DXGI: Failed to get adapter for present device");
     
-    // Set initial window mode and fullscreen state
+    // Apply initial window mode and fullscreen state
     if (!m_descFs.Windowed && FAILED(EnterFullscreenMode(nullptr)))
       throw DxvkError("DXGI: Failed to set initial fullscreen state");
   }
   
   
   DxgiSwapChain::~DxgiSwapChain() {
-    Com<IDXGIOutput> output;
+    RestoreDisplayMode(m_monitor);
+
+    // Decouple swap chain from monitor if necessary
+    DXGI_VK_MONITOR_DATA* monitorInfo = nullptr;
     
-    if (SUCCEEDED(m_adapter->GetOutputFromMonitor(m_monitor, &output)))
-      RestoreDisplayMode(output.ptr());
+    if (SUCCEEDED(AcquireMonitorData(m_monitor, &monitorInfo))) {
+      if (monitorInfo->pSwapChain == this)
+        monitorInfo->pSwapChain = nullptr;
+      
+      ReleaseMonitorData();
+    }
   }
   
   
@@ -105,7 +101,7 @@ namespace dxvk {
         (windowRect.top + windowRect.bottom) / 2 },
       MONITOR_DEFAULTTOPRIMARY);
     
-    return m_adapter->GetOutputFromMonitor(monitor, ppOutput);
+    return GetOutputFromMonitor(monitor, ppOutput);
   }
   
   
@@ -186,7 +182,7 @@ namespace dxvk {
       *ppTarget = nullptr;
       
       if (!m_descFs.Windowed)
-        hr = m_adapter->GetOutputFromMonitor(m_monitor, ppTarget);
+        hr = GetOutputFromMonitor(m_monitor, ppTarget);
     }
     
     return hr;
@@ -264,17 +260,6 @@ namespace dxvk {
     SyncInterval = std::min<UINT>(SyncInterval, 4);
 
     try {
-      // If in fullscreen mode, apply any updated gamma curve
-      // if it has been changed since the last present call.
-      DXGI_VK_OUTPUT_DATA outputData;
-      
-      if (SUCCEEDED(m_adapter->GetOutputData(m_monitor, &outputData)) && outputData.GammaDirty) {
-        SetGammaControl(&outputData.GammaCurve);
-        
-        outputData.GammaDirty = FALSE;
-        m_adapter->SetOutputData(m_monitor, &outputData);
-      }
-      
       return m_presenter->Present(SyncInterval, PresentFlags, nullptr);
     } catch (const DxvkError& err) {
       Logger::err(err.message());
@@ -292,11 +277,13 @@ namespace dxvk {
     if (!IsWindow(m_window))
       return DXGI_ERROR_INVALID_CALL;
     
-    const VkExtent2D windowSize = GetWindowSize();
-    
     std::lock_guard<std::mutex> lock(m_lockBuffer);
-    m_desc.Width  = Width  != 0 ? Width  : windowSize.width;
-    m_desc.Height = Height != 0 ? Height : windowSize.height;
+    m_desc.Width  = Width;
+    m_desc.Height = Height;
+    
+    GetWindowClientSize(m_window,
+      m_desc.Width  ? nullptr : &m_desc.Width,
+      m_desc.Height ? nullptr : &m_desc.Height);
     
     if (BufferCount != 0)
       m_desc.BufferCount = BufferCount;
@@ -359,7 +346,7 @@ namespace dxvk {
     } else {
       Com<IDXGIOutput> output;
       
-      if (FAILED(m_adapter->GetOutputFromMonitor(m_monitor, &output))) {
+      if (FAILED(GetOutputFromMonitor(m_monitor, &output))) {
         Logger::err("DXGI: ResizeTarget: Failed to query containing output");
         return E_FAIL;
       }
@@ -482,35 +469,20 @@ namespace dxvk {
   }
 
 
-  HRESULT DxgiSwapChain::SetColorSpace1(DXGI_COLOR_SPACE_TYPE ColorSpace) {
+  HRESULT STDMETHODCALLTYPE DxgiSwapChain::SetColorSpace1(DXGI_COLOR_SPACE_TYPE ColorSpace) {
     Logger::err("DxgiSwapChain::SetColorSpace1: Not implemented");
     return E_NOTIMPL;
   }
+  
+  
+  HRESULT STDMETHODCALLTYPE DxgiSwapChain::SetGammaControl(
+          UINT                      NumPoints,
+    const DXGI_RGB*                 pGammaCurve) {
+    std::lock_guard<std::mutex> lockBuf(m_lockBuffer);
+    return m_presenter->SetGammaControl(NumPoints, pGammaCurve);
+  }
 
 
-  HRESULT DxgiSwapChain::SetGammaControl(const DXGI_GAMMA_CONTROL* pGammaControl) {
-    return m_presenter->SetGammaControl(DXGI_VK_GAMMA_CP_COUNT, pGammaControl->GammaCurve);
-  }
-  
-  
-  HRESULT DxgiSwapChain::SetDefaultGammaControl() {
-    return m_presenter->SetGammaControl(0, nullptr);
-  }
-  
-  
-  VkExtent2D DxgiSwapChain::GetWindowSize() const {
-    RECT windowRect;
-    
-    if (!::GetClientRect(m_window, &windowRect))
-      windowRect = RECT();
-    
-    VkExtent2D result;
-    result.width  = windowRect.right;
-    result.height = windowRect.bottom;
-    return result;
-  }
-  
-  
   HRESULT DxgiSwapChain::EnterFullscreenMode(IDXGIOutput* pTarget) {
     Com<IDXGIOutput> output = static_cast<DxgiOutput*>(pTarget);
 
@@ -533,8 +505,10 @@ namespace dxvk {
       displayMode.Height           = m_desc.Height;
       displayMode.RefreshRate      = m_descFs.RefreshRate;
       displayMode.Format           = m_desc.Format;
-      displayMode.ScanlineOrdering = m_descFs.ScanlineOrdering;
-      displayMode.Scaling          = m_descFs.Scaling;
+      // Ignore these two, games usually use them wrong and we don't
+      // support any scaling modes except UNSPECIFIED anyway.
+      displayMode.ScanlineOrdering = DXGI_MODE_SCANLINE_ORDER_UNSPECIFIED;
+      displayMode.Scaling          = DXGI_MODE_SCALING_UNSPECIFIED;
       
       if (FAILED(ChangeDisplayMode(output.ptr(), &displayMode))) {
         Logger::err("DXGI: EnterFullscreenMode: Failed to change display mode");
@@ -569,20 +543,40 @@ namespace dxvk {
       SWP_FRAMECHANGED | SWP_SHOWWINDOW | SWP_NOACTIVATE);
     
     m_monitor = desc.Monitor;
+
+    // Apply current gamma curve of the output
+    DXGI_VK_MONITOR_DATA* monitorInfo = nullptr;
+
+    if (SUCCEEDED(AcquireMonitorData(m_monitor, &monitorInfo))) {
+      if (!monitorInfo->pSwapChain)
+        monitorInfo->pSwapChain = this;
+      
+      SetGammaControl(DXGI_VK_GAMMA_CP_COUNT, monitorInfo->GammaCurve.GammaCurve);
+      ReleaseMonitorData();
+    }
+
     return S_OK;
   }
   
   
   HRESULT DxgiSwapChain::LeaveFullscreenMode() {
-    Com<IDXGIOutput> output;
-
     if (!IsWindow(m_window))
       return DXGI_ERROR_NOT_CURRENTLY_AVAILABLE;
     
-    if (FAILED(m_adapter->GetOutputFromMonitor(m_monitor, &output))
-     || FAILED(RestoreDisplayMode(output.ptr())))
+    if (FAILED(RestoreDisplayMode(m_monitor)))
       Logger::warn("DXGI: LeaveFullscreenMode: Failed to restore display mode");
     
+    // Reset gamma control and decouple swap chain from monitor
+    DXGI_VK_MONITOR_DATA* monitorInfo = nullptr;
+
+    if (SUCCEEDED(AcquireMonitorData(m_monitor, &monitorInfo))) {
+      if (monitorInfo->pSwapChain == this)
+        monitorInfo->pSwapChain = nullptr;
+      
+      SetGammaControl(0, nullptr);
+      ReleaseMonitorData();
+    }
+    
     // Restore internal state
     m_descFs.Windowed = TRUE;
     m_monitor = nullptr;
@@ -605,26 +599,27 @@ namespace dxvk {
       rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top,
       SWP_FRAMECHANGED | SWP_NOZORDER | SWP_NOACTIVATE);
     
-    return SetDefaultGammaControl();
+    return S_OK;
   }
   
   
   HRESULT DxgiSwapChain::ChangeDisplayMode(
           IDXGIOutput*            pOutput,
     const DXGI_MODE_DESC*         pDisplayMode) {
-    auto output = static_cast<DxgiOutput*>(pOutput);
-
-    if (output == nullptr)
+    if (!pOutput)
       return DXGI_ERROR_INVALID_CALL;
     
     // Find a mode that the output supports
+    DXGI_OUTPUT_DESC outputDesc;
+    pOutput->GetDesc(&outputDesc);
+    
     DXGI_MODE_DESC preferredMode = *pDisplayMode;
     DXGI_MODE_DESC selectedMode;
 
     if (preferredMode.Format == DXGI_FORMAT_UNKNOWN)
       preferredMode.Format = m_desc.Format;
     
-    HRESULT hr = output->FindClosestMatchingMode(
+    HRESULT hr = pOutput->FindClosestMatchingMode(
       &preferredMode, &selectedMode, nullptr);
     
     if (FAILED(hr)) {
@@ -636,26 +631,24 @@ namespace dxvk {
       return hr;
     }
     
-    return output->SetDisplayMode(&selectedMode);
+    return SetMonitorDisplayMode(outputDesc.Monitor, &selectedMode);
   }
   
   
-  HRESULT DxgiSwapChain::RestoreDisplayMode(IDXGIOutput* pOutput) {
-    auto output = static_cast<DxgiOutput*>(pOutput);
-    
-    if (output == nullptr)
+  HRESULT DxgiSwapChain::RestoreDisplayMode(HMONITOR hMonitor) {
+    if (!hMonitor)
       return DXGI_ERROR_INVALID_CALL;
     
     // Restore registry settings
     DXGI_MODE_DESC mode;
     
-    HRESULT hr = output->GetDisplayMode(
-      &mode, ENUM_REGISTRY_SETTINGS);
+    HRESULT hr = GetMonitorDisplayMode(
+      hMonitor, ENUM_REGISTRY_SETTINGS, &mode);
     
     if (FAILED(hr))
       return hr;
     
-    return output->SetDisplayMode(&mode);
+    return SetMonitorDisplayMode(hMonitor, &mode);
   }
   
   
@@ -672,18 +665,73 @@ namespace dxvk {
   }
 
 
-  HRESULT DxgiSwapChain::CreatePresenter(
-            IUnknown*               pDevice,
-            IDXGIVkSwapChain**      ppSwapChain) {
-    Com<IDXGIVkPresentDevice> presentDevice;
-
-    // Retrieve a device pointer that allows us to
-    // communicate with the underlying D3D device
-    if (SUCCEEDED(pDevice->QueryInterface(__uuidof(IDXGIVkPresentDevice),
-        reinterpret_cast<void**>(&presentDevice))))
-      return presentDevice->CreateSwapChainForHwnd(m_window, &m_desc, ppSwapChain);
+  HRESULT DxgiSwapChain::GetOutputFromMonitor(
+          HMONITOR                  Monitor,
+          IDXGIOutput**             ppOutput) {
+    if (!ppOutput)
+      return DXGI_ERROR_INVALID_CALL;
     
-    return E_INVALIDARG;
+    for (uint32_t i = 0; SUCCEEDED(m_adapter->EnumOutputs(i, ppOutput)); i++) {
+      DXGI_OUTPUT_DESC outputDesc;
+      (*ppOutput)->GetDesc(&outputDesc);
+      
+      if (outputDesc.Monitor == Monitor)
+        return S_OK;
+      
+      (*ppOutput)->Release();
+      (*ppOutput) = nullptr;
+    }
+    
+    return DXGI_ERROR_NOT_FOUND;
+  }
+  
+  
+  HRESULT CreateDxvkSwapChainForHwnd(
+          IDXGIFactory*             pFactory,
+          IDXGIVkPresentDevice*     pDevice,
+          HWND                      hWnd,
+    const DXGI_SWAP_CHAIN_DESC1*    pDesc,
+    const DXGI_SWAP_CHAIN_FULLSCREEN_DESC* pFullscreenDesc,
+          IDXGIOutput*              pRestrictToOutput,
+          IDXGISwapChain1**         ppSwapChain) {
+    // Make sure the back buffer size is not zero
+    DXGI_SWAP_CHAIN_DESC1 desc = *pDesc;
+    
+    GetWindowClientSize(hWnd,
+      desc.Width  ? nullptr : &desc.Width,
+      desc.Height ? nullptr : &desc.Height);
+    
+    // If necessary, set up a default set of
+    // fullscreen parameters for the swap chain
+    DXGI_SWAP_CHAIN_FULLSCREEN_DESC fsDesc;
+    
+    if (pFullscreenDesc) {
+      fsDesc = *pFullscreenDesc;
+    } else {
+      fsDesc.RefreshRate      = { 0, 0 };
+      fsDesc.ScanlineOrdering = DXGI_MODE_SCANLINE_ORDER_UNSPECIFIED;
+      fsDesc.Scaling          = DXGI_MODE_SCALING_UNSPECIFIED;
+      fsDesc.Windowed         = TRUE;
+    }
+    
+    // Create presenter for the device
+    Com<IDXGIVkSwapChain> presenter;
+    
+    HRESULT hr = pDevice->CreateSwapChainForHwnd(
+      hWnd, &desc, &presenter);
+    
+    if (FAILED(hr))
+      return hr;
+    
+    try {
+      // Create actual swap chain object
+      *ppSwapChain = ref(new DxgiSwapChain(
+        pFactory, presenter.ptr(), hWnd, &desc, &fsDesc));
+      return S_OK;
+    } catch (const DxvkError& e) {
+      Logger::err(e.message());
+      return DXGI_ERROR_UNSUPPORTED;
+    }
   }
   
 }
