@@ -8,7 +8,6 @@ namespace dxvk {
   
   class DxvkDevice;
   class DxvkInstance;
-  class DxvkSurface;
   
   /**
    * \brief GPU vendors
@@ -183,6 +182,17 @@ namespace dxvk {
     bool checkFeatureSupport(
       const DxvkDeviceFeatures& required) const;
     
+    /**
+     * \brief Enables extensions for this adapter
+     *
+     * When creating a device, all extensions that
+     * are added using this method will be enabled
+     * in addition to the ones required by DXVK.
+     * This is used for OpenVR support.
+     */
+    void enableExtensions(
+      const DxvkNameSet&        extensions);
+    
     /**
      * \brief Creates a DXVK device
      * 
@@ -193,17 +203,6 @@ namespace dxvk {
     Rc<DxvkDevice> createDevice(
             DxvkDeviceFeatures  enabledFeatures);
     
-    /**
-     * \brief Creates a surface
-     * 
-     * \param [in] instance Module instance
-     * \param [in] window Application window
-     * \returns Surface handle
-     */
-    Rc<DxvkSurface> createSurface(
-      HINSTANCE instance,
-      HWND      window);
-    
     /**
      * \brief Registers memory allocation
      * 
@@ -226,6 +225,22 @@ namespace dxvk {
             uint32_t            heap,
             VkDeviceSize        bytes);
     
+    /**
+     * \brief Tests if the driver matches certain criteria
+     *
+     * \param [in] vendor GPU vendor
+     * \param [in] driver Driver. Ignored when the
+     *    driver properties extension is not supported.
+     * \param [in] minVer Match versions starting with this one
+     * \param [in] maxVer Match versions lower than this one
+     * \returns \c True if the driver matches these criteria
+     */
+    bool matchesDriver(
+            DxvkGpuVendor       vendor,
+            VkDriverIdKHR       driver,
+            uint32_t            minVer,
+            uint32_t            maxVer) const;
+    
     /**
      * \brief Logs DXVK adapter info
      * 
@@ -240,6 +255,7 @@ namespace dxvk {
     Rc<vk::InstanceFn>  m_vki;
     VkPhysicalDevice    m_handle;
 
+    DxvkNameSet         m_extraExtensions;
     DxvkNameSet         m_deviceExtensions;
     DxvkDeviceInfo      m_deviceInfo;
     DxvkDeviceFeatures  m_deviceFeatures;
@@ -254,8 +270,6 @@ namespace dxvk {
     void queryDeviceFeatures();
     void queryDeviceQueues();
     
-    uint32_t getAdapterIndex() const;
-
     static void logNameList(const DxvkNameList& names);
     
   };
