@@ -6,7 +6,7 @@ namespace dxvk {
   DxvkBarrierSet::~DxvkBarrierSet() { }
   
   void DxvkBarrierSet::accessBuffer(
-    const DxvkPhysicalBufferSlice&  bufSlice,
+    const DxvkBufferSliceHandle&    bufSlice,
           VkPipelineStageFlags      srcStages,
           VkAccessFlags             srcAccess,
           VkPipelineStageFlags      dstStages,
@@ -16,6 +16,10 @@ namespace dxvk {
     m_srcStages |= srcStages;
     m_dstStages |= dstStages;
     
+    if (srcStages == VK_PIPELINE_STAGE_BOTTOM_OF_PIPE_BIT
+     || dstStages == VK_PIPELINE_STAGE_TOP_OF_PIPE_BIT)
+      access.set(DxvkAccess::Write);
+    
     if (access.test(DxvkAccess::Write)) {
       VkBufferMemoryBarrier barrier;
       barrier.sType               = VK_STRUCTURE_TYPE_BUFFER_MEMORY_BARRIER;
@@ -24,9 +28,9 @@ namespace dxvk {
       barrier.dstAccessMask       = dstAccess;
       barrier.srcQueueFamilyIndex = VK_QUEUE_FAMILY_IGNORED;
       barrier.dstQueueFamilyIndex = VK_QUEUE_FAMILY_IGNORED;
-      barrier.buffer              = bufSlice.handle();
-      barrier.offset              = bufSlice.offset();
-      barrier.size                = bufSlice.length();
+      barrier.buffer              = bufSlice.handle;
+      barrier.offset              = bufSlice.offset;
+      barrier.size                = bufSlice.length;
       m_bufBarriers.push_back(barrier);
     }
 
@@ -48,7 +52,9 @@ namespace dxvk {
     m_srcStages |= srcStages;
     m_dstStages |= dstStages;
     
-    if (srcLayout != dstLayout)
+    if (srcStages == VK_PIPELINE_STAGE_BOTTOM_OF_PIPE_BIT
+     || dstStages == VK_PIPELINE_STAGE_TOP_OF_PIPE_BIT
+     || srcLayout != dstLayout)
       access.set(DxvkAccess::Write);
     
     if (access.test(DxvkAccess::Write)) {
@@ -69,16 +75,32 @@ namespace dxvk {
 
     m_imgSlices.push_back({ image.ptr(), subresources, access });
   }
+
+
+  void DxvkBarrierSet::accessMemory(
+          VkPipelineStageFlags      srcStages,
+          VkAccessFlags             srcAccess,
+          VkPipelineStageFlags      dstStages,
+          VkAccessFlags             dstAccess) {
+    m_srcStages |= srcStages;
+    m_dstStages |= dstStages;
+
+    m_srcAccess |= srcAccess;
+    m_dstAccess |= dstAccess;
+  }
   
   
   bool DxvkBarrierSet::isBufferDirty(
-    const DxvkPhysicalBufferSlice&  bufSlice,
+    const DxvkBufferSliceHandle&    bufSlice,
           DxvkAccessFlags           bufAccess) {
-    bool result = false;
+    bool result = m_srcAccess || m_dstAccess;
 
     for (uint32_t i = 0; i < m_bufSlices.size() && !result; i++) {
-      result = (bufSlice.overlaps(m_bufSlices[i].slice))
-            && (bufAccess | m_bufSlices[i].access).test(DxvkAccess::Write);
+      const DxvkBufferSliceHandle& dstSlice = m_bufSlices[i].slice;
+
+      result = (bufSlice.handle == dstSlice.handle) && (bufAccess | m_bufSlices[i].access).test(DxvkAccess::Write)
+            && (bufSlice.offset + bufSlice.length > dstSlice.offset)
+            && (bufSlice.offset < dstSlice.offset + dstSlice.length);
     }
 
     return result;
@@ -89,16 +111,17 @@ namespace dxvk {
     const Rc<DxvkImage>&            image,
     const VkImageSubresourceRange&  imgSubres,
           DxvkAccessFlags           imgAccess) {
-    bool result = false;
+    bool result = (m_srcStages & image->info().stages)
+               && (m_srcAccess & image->info().access);
 
     for (uint32_t i = 0; i < m_imgSlices.size() && !result; i++) {
       const VkImageSubresourceRange& dstSubres = m_imgSlices[i].subres;
 
       result = (image == m_imgSlices[i].image) && (imgAccess | m_imgSlices[i].access).test(DxvkAccess::Write)
-            && imgSubres.baseArrayLayer < dstSubres.baseArrayLayer + dstSubres.layerCount
-            && imgSubres.baseArrayLayer + imgSubres.layerCount     > dstSubres.baseArrayLayer
-            && imgSubres.baseMipLevel   < dstSubres.baseMipLevel   + dstSubres.levelCount
-            && imgSubres.baseMipLevel   + imgSubres.levelCount     > dstSubres.baseMipLevel;
+            && (imgSubres.baseArrayLayer < dstSubres.baseArrayLayer + dstSubres.layerCount)
+            && (imgSubres.baseArrayLayer + imgSubres.layerCount     > dstSubres.baseArrayLayer)
+            && (imgSubres.baseMipLevel   < dstSubres.baseMipLevel   + dstSubres.levelCount)
+            && (imgSubres.baseMipLevel   + imgSubres.levelCount     > dstSubres.baseMipLevel);
     }
 
     return result;
@@ -106,16 +129,26 @@ namespace dxvk {
   
 
   void DxvkBarrierSet::recordCommands(const Rc<DxvkCommandList>& commandList) {
-    if ((m_srcStages | m_dstStages) != 0) {
+    if (m_srcStages | m_dstStages) {
       VkPipelineStageFlags srcFlags = m_srcStages;
       VkPipelineStageFlags dstFlags = m_dstStages;
       
-      if (srcFlags == 0) srcFlags = VK_PIPELINE_STAGE_TOP_OF_PIPE_BIT;
-      if (dstFlags == 0) dstFlags = VK_PIPELINE_STAGE_BOTTOM_OF_PIPE_BIT;
+      if (!srcFlags) srcFlags = VK_PIPELINE_STAGE_TOP_OF_PIPE_BIT;
+      if (!dstFlags) dstFlags = VK_PIPELINE_STAGE_BOTTOM_OF_PIPE_BIT;
+
+      VkMemoryBarrier memBarrier;
+      memBarrier.sType = VK_STRUCTURE_TYPE_MEMORY_BARRIER;
+      memBarrier.pNext = nullptr;
+      memBarrier.srcAccessMask = m_srcAccess;
+      memBarrier.dstAccessMask = m_dstAccess;
+
+      VkMemoryBarrier* pMemBarrier = nullptr;
+      if (m_srcAccess | m_dstAccess)
+        pMemBarrier = &memBarrier;
       
       commandList->cmdPipelineBarrier(
         srcFlags, dstFlags, 0,
-        m_memBarriers.size(), m_memBarriers.data(),
+        pMemBarrier ? 1 : 0, pMemBarrier,
         m_bufBarriers.size(), m_bufBarriers.data(),
         m_imgBarriers.size(), m_imgBarriers.data());
       
@@ -128,7 +161,6 @@ namespace dxvk {
     m_srcStages = 0;
     m_dstStages = 0;
     
-    m_memBarriers.resize(0);
     m_bufBarriers.resize(0);
     m_imgBarriers.resize(0);
 
