@@ -12,7 +12,6 @@
 #include "dxvk_query_tracker.h"
 #include "dxvk_staging.h"
 #include "dxvk_stats.h"
-#include "dxvk_sync.h"
 
 namespace dxvk {
   
@@ -109,17 +108,17 @@ namespace dxvk {
     void endRecording();
     
     /**
-     * \brief Frees physical buffer slice
+     * \brief Frees buffer slice
      * 
      * After the command buffer execution has finished,
-     * the given physical slice will be released to the
+     * the given buffer slice will be released to the
      * virtual buffer object so that it can be reused.
      * \param [in] buffer The virtual buffer object
-     * \param [in] slice The physical buffer slice
+     * \param [in] slice The buffer slice handle
      */
-    void freePhysicalBufferSlice(
+    void freeBufferSlice(
       const Rc<DxvkBuffer>&           buffer,
-      const DxvkPhysicalBufferSlice&  slice) {
+      const DxvkBufferSliceHandle&    slice) {
       m_bufferTracker.freeBufferSlice(buffer, slice);
     }
     
@@ -156,6 +155,14 @@ namespace dxvk {
     void trackEvent(const DxvkEventRevision& event) {
       m_eventTracker.trackEvent(event);
     }
+
+    /**
+     * \brief Tracks a descriptor pool
+     * \param [in] pool The descriptor pool
+     */
+    void trackDescriptorPool(Rc<DxvkDescriptorPool> pool) {
+      m_descriptorPoolTracker.trackDescriptorPool(pool);
+    }
     
     /**
      * \brief Signals tracked events
@@ -188,12 +195,6 @@ namespace dxvk {
      */
     void reset();
     
-    VkDescriptorSet allocateDescriptorSet(
-            VkDescriptorSetLayout   descriptorLayout) {
-      return m_descAlloc.alloc(descriptorLayout);
-    }
-    
-    
     void updateDescriptorSets(
             uint32_t                      descriptorWriteCount,
       const VkWriteDescriptorSet*         pDescriptorWrites) {
@@ -656,7 +657,7 @@ namespace dxvk {
     
     DxvkCmdBufferFlags  m_cmdBuffersUsed;
     DxvkLifetimeTracker m_resources;
-    DxvkDescriptorAlloc m_descAlloc;
+    DxvkDescriptorPoolTracker m_descriptorPoolTracker;
     DxvkStagingAlloc    m_stagingAlloc;
     DxvkQueryTracker    m_queryTracker;
     DxvkEventTracker    m_eventTracker;
