@@ -575,11 +575,11 @@ namespace dxvk {
      * \warning If the buffer is used by another context,
      * invalidating it will result in undefined behaviour.
      * \param [in] buffer The buffer to invalidate
-     * \param [in] slice New physical buffer slice
+     * \param [in] slice New buffer slice handle
      */
     void invalidateBuffer(
       const Rc<DxvkBuffer>&           buffer,
-      const DxvkPhysicalBufferSlice&  slice);
+      const DxvkBufferSliceHandle&    slice);
     
     /**
      * \brief Resolves a multisampled image resource
@@ -674,7 +674,17 @@ namespace dxvk {
      * \param [in] blendConstants Blend constants
      */
     void setBlendConstants(
-      const DxvkBlendConstants& blendConstants);
+            DxvkBlendConstants  blendConstants);
+    
+    /**
+     * \brief Sets depth bias
+     * 
+     * Depth bias has to be enabled explicitly in
+     * the rasterizer state to have any effect.
+     * \param [in] depthBias Depth bias values
+     */
+    void setDepthBias(
+            DxvkDepthBias       depthBias);
     
     /**
      * \brief Sets stencil reference
@@ -683,7 +693,7 @@ namespace dxvk {
      * \param [in] reference Reference value
      */
     void setStencilReference(
-      const uint32_t            reference);
+            uint32_t            reference);
     
     /**
      * \brief Sets input assembly state
@@ -768,7 +778,9 @@ namespace dxvk {
     const Rc<DxvkMetaPackObjects>     m_metaPack;
     const Rc<DxvkMetaResolveObjects>  m_metaResolve;
     
-    Rc<DxvkCommandList> m_cmd;
+    Rc<DxvkCommandList>     m_cmd;
+    Rc<DxvkDescriptorPool>  m_descPool;
+
     DxvkContextFlags    m_flags;
     DxvkContextState    m_state;
 
@@ -832,6 +844,7 @@ namespace dxvk {
     
     void startRenderPass();
     void spillRenderPass();
+    void clearRenderPass();
     
     void renderPassBindFramebuffer(
       const Rc<DxvkFramebuffer>&  framebuffer,
@@ -890,7 +903,7 @@ namespace dxvk {
     bool validateGraphicsState();
     
     void commitComputeState();
-    void commitGraphicsState();
+    void commitGraphicsState(bool indexed);
     
     void commitComputeInitBarriers();
     void commitComputePostBarriers();
@@ -902,6 +915,9 @@ namespace dxvk {
             VkAccessFlags             srcAccess,
             VkPipelineStageFlags      dstStages,
             VkAccessFlags             dstAccess);
+    
+    VkDescriptorSet allocateDescriptorSet(
+            VkDescriptorSetLayout     layout);
 
     void trackDrawBuffer();
     
