@@ -3,7 +3,7 @@
 namespace dxvk {
   
   DxvkDeviceFilter::DxvkDeviceFilter() {
-    m_matchDeviceName = env::getEnvVar(L"DXVK_FILTER_DEVICE_NAME");
+    m_matchDeviceName = env::getEnvVar("DXVK_FILTER_DEVICE_NAME");
     
     if (m_matchDeviceName.size() != 0)
       m_flags.set(DxvkDeviceFilterFlag::MatchDeviceName);
