@@ -52,5 +52,33 @@ namespace dxvk {
   DxvkEvent::Status DxvkEvent::unpack(uint64_t packed) {
     return { DxvkEventStatus(packed >> 32), uint32_t(packed) };
   }
+
+
+
+  
+  DxvkEventTracker::DxvkEventTracker() {
+    
+  }
+  
+  
+  DxvkEventTracker::~DxvkEventTracker() {
+    
+  }
+  
+  
+  void DxvkEventTracker::trackEvent(const DxvkEventRevision& event) {
+    m_events.push_back(event);
+  }
+  
+  
+  void DxvkEventTracker::signalEvents() {
+    for (const DxvkEventRevision& event : m_events)
+      event.event->signal(event.revision);
+  }
+  
+  
+  void DxvkEventTracker::reset() {
+    m_events.clear();
+  }
   
 }
\ No newline at end of file
