@@ -122,7 +122,6 @@ namespace dxvk {
     // applications may not have OpenVR loaded at the time
     // they create the DXGI instance, so we try our own DLL.
     m_ovrApi = this->loadLibrary();
-    m_loadedOvrApi = m_ovrApi != nullptr;
     
     if (!m_ovrApi) {
       Logger::warn("OpenVR: Failed to locate module");
@@ -195,9 +194,12 @@ namespace dxvk {
     // on winelib, load native openvr_api directly
     if (!(handle = ::dlopen("libopenvr_api.so", RTLD_LAZY | RTLD_NOLOAD)))
       handle = ::dlopen("libopenvr_api_dxvk.so", RTLD_LAZY | RTLD_LOCAL);
+    m_loadedOvrApi = handle != nullptr;
     #else
-    if (!(handle = ::GetModuleHandle("openvr_api.dll")))
+    if (!(handle = ::GetModuleHandle("openvr_api.dll"))) {
       handle = ::LoadLibrary("openvr_api_dxvk.dll");
+      m_loadedOvrApi = handle != nullptr;
+    }
     #endif
     return handle;
   }
