@@ -12,9 +12,16 @@ namespace dxvk {
     /// a heap than the device supports.
     bool allowMemoryOvercommit;
 
+    /// Enable state cache
+    bool enableStateCache;
+
     /// Number of compiler threads
     /// when using the state cache
     int32_t numCompilerThreads;
+
+    /// Shader-related options
+    Tristate useRawSsbo;
+    Tristate useEarlyDiscard;
   };
 
 }
\ No newline at end of file
