@@ -755,6 +755,21 @@ namespace dxvk {
       m_code.putInt32(indexArray[i]);
     return resultId;
   }
+
+
+  uint32_t SpirvModule::opArrayLength(
+          uint32_t                resultType,
+          uint32_t                structure,
+          uint32_t                memberId) {
+    uint32_t resultId = this->allocateId();
+    
+    m_code.putIns (spv::OpArrayLength, 5);
+    m_code.putWord(resultType);
+    m_code.putWord(resultId);
+    m_code.putWord(structure);
+    m_code.putWord(memberId);
+    return resultId;
+  }
   
   
   uint32_t SpirvModule::opAny(
@@ -2872,6 +2887,38 @@ namespace dxvk {
   }
   
   
+  uint32_t SpirvModule::opGroupNonUniformBallot(
+          uint32_t                resultType,
+          uint32_t                execution,
+          uint32_t                predicate) {
+    uint32_t resultId = this->allocateId();
+
+    m_code.putIns(spv::OpGroupNonUniformBallot, 5);
+    m_code.putWord(resultType);
+    m_code.putWord(resultId);
+    m_code.putWord(execution);
+    m_code.putWord(predicate);
+    return resultId;
+  }
+
+  
+  uint32_t SpirvModule::opGroupNonUniformBallotBitCount(
+          uint32_t                resultType,
+          uint32_t                execution,
+          uint32_t                operation,
+          uint32_t                ballot) {
+    uint32_t resultId = this->allocateId();
+
+    m_code.putIns(spv::OpGroupNonUniformBallotBitCount, 6);
+    m_code.putWord(resultType);
+    m_code.putWord(resultId);
+    m_code.putWord(execution);
+    m_code.putWord(operation);
+    m_code.putWord(ballot);
+    return resultId;
+  }
+
+
   uint32_t SpirvModule::opGroupNonUniformLogicalAnd(
           uint32_t                resultType,
           uint32_t                execution,
