@@ -308,6 +308,11 @@ namespace dxvk {
             uint32_t                indexCount,
       const uint32_t*               indexArray);
     
+    uint32_t opArrayLength(
+            uint32_t                resultType,
+            uint32_t                structure,
+            uint32_t                memberId);
+    
     uint32_t opAny(
             uint32_t                resultType,
             uint32_t                vector);
@@ -992,6 +997,17 @@ namespace dxvk {
             uint32_t                reference,
       const SpirvImageOperands&     operands);
 
+    uint32_t opGroupNonUniformBallot(
+            uint32_t                resultType,
+            uint32_t                execution,
+            uint32_t                predicate);
+    
+    uint32_t opGroupNonUniformBallotBitCount(
+            uint32_t                resultType,
+            uint32_t                execution,
+            uint32_t                operation,
+            uint32_t                ballot);
+    
     uint32_t opGroupNonUniformLogicalAnd(
             uint32_t                resultType,
             uint32_t                execution,
