@@ -11,7 +11,7 @@
 namespace dxvk {
 
   const static std::unordered_map<std::string, Config> g_appDefaults = {{
-    /* Assassin's Creed Syndicate - amdags issues */
+    /* Assassin's Creed Syndicate: amdags issues  */
     { "ACS.exe", {{
       { "dxgi.customVendorId",              "10de" },
     }} },
@@ -19,7 +19,8 @@ namespace dxvk {
     { "Dishonored2.exe", {{
       { "d3d11.allowMapFlagNoWait",         "True" }
     }} },
-    /* Elite Dangerous                            */
+    /* Elite Dangerous: Compiles weird shaders    *
+     * when running on AMD hardware               */
     { "EliteDangerous64.exe", {{
       { "dxgi.customVendorId",              "10de" },
     }} },
@@ -27,7 +28,8 @@ namespace dxvk {
     { "EthanCarter-Win64-Shipping.exe", {{
       { "dxgi.customVendorId",              "10de" },
     }} },
-    /* The Evil Within                            */
+    /* The Evil Within: Submits command lists     * 
+     * multiple times                             */
     { "EvilWithin.exe", {{
       { "d3d11.dcSingleUseMode",            "False" },
     }} },
@@ -35,34 +37,45 @@ namespace dxvk {
     { "EvilWithinDemo.exe", {{
       { "d3d11.dcSingleUseMode",            "False" },
     }} },
-    /* F1 2015                                    */
-    { "F1_2015.exe", {{
-      { "d3d11.fakeStreamOutSupport",       "True" },
-    }} },
     /* Far Cry 5                                  */
     { "FarCry5.exe", {{
       { "d3d11.allowMapFlagNoWait",         "True" }
     }} },
-    /* Final Fantasy XV                           */
-    { "ffxv_s.exe", {{
-      { "d3d11.fakeStreamOutSupport",       "True" },
-    }} },
-    /* Frostpunk                                  */
+    /* Frostpunk: Renders one frame with D3D9     *
+     * after creating the DXGI swap chain         */
     { "Frostpunk.exe", {{
       { "dxgi.deferSurfaceCreation",        "True" },
     }} },
-    /* Mafia 3                                    */
-    { "mafia3.exe", {{
-      { "d3d11.fakeStreamOutSupport",       "True" },
-    }} },
-    /* Overwatch                                  */
-    { "Overwatch.exe", {{
-      { "d3d11.fakeStreamOutSupport",       "True" },
-    }} },
-    /* Quantum Break                              */
+    /* Quantum Break: Mever initializes shared    *
+     * memory in one of its compute shaders       */
     { "QuantumBreak.exe", {{
       { "d3d11.zeroInitWorkgroupMemory",    "True" },
     }} },
+    /* Anno 2205: Random crashes with state cache */
+    { "anno2205.exe", {{
+      { "dxvk.enableStateCache",            "False" },
+    }} },
+    /* Fifa '19: Binds typed buffer SRV to shader *
+     * that expects raw/structured buffer SRV     */
+    { "FIFA19.exe", {{
+      { "dxvk.useRawSsbo",                  "True" },
+    }} },
+    /* Fifa '19 Demo                              */
+    { "FIFA19_demo.exe", {{
+      { "dxvk.useRawSsbo",                  "True" },
+    }} },
+    /* Call of Duty WW2                           */
+    { "s2_sp64_ship.exe", {{
+      { "dxgi.nvapiHack",                   "False" },
+    }} },
+    /* Need for Speed 2015                        */
+    { "NFS16.exe", {{
+      { "dxgi.nvapiHack",                   "False" },
+    }} },
+    /* Mass Effect Andromeda                      */
+    { "MassEffectAndromeda.exe", {{
+      { "dxgi.nvapiHack",                   "False" },
+    }} },
   }};
 
 
@@ -189,6 +202,24 @@ namespace dxvk {
     result = sign * intval;
     return true;
   }
+  
+  
+  bool Config::parseOptionValue(
+    const std::string&  value,
+          Tristate&     result) {
+    if (value == "True") {
+      result = Tristate::True;
+      return true;
+    } else if (value == "False") {
+      result = Tristate::False;
+      return true;
+    } else if (value == "Auto") {
+      result = Tristate::Auto;
+      return true;
+    } else {
+      return false;
+    }
+  }
 
 
   Config Config::getAppConfig(const std::string& appName) {
